#ifndef GEOMETRY
#define GEOMETRY

#include <QPolygonF>

QRectF addBBox(QRectF r1, QRectF r2);
bool isInBox(const QPointF &p, const QRectF &box);

class ConvexPolygon:public QPolygonF
{
public:
	ConvexPolygon ();
	ConvexPolygon (QPolygonF p);
	void calcCentroid() ;
	QPointF centroid() const;
	qreal weight() const;
private:
	QPointF _centroid;
	qreal _area;
};

QPointF normalize (const QPointF &p);


qreal dotProduct (const QPointF &a, const QPointF &b);

QPointF scale  (const QPointF &v,const qreal &f);
QPointF invert (const QPointF &v);

class PolygonCollisionResult {
public:
    // Are the polygons going to intersect forward in time?
    bool willIntersect;

    // Are the polygons currently intersecting?
    bool intersect;

    // The translation to apply to the first polygon to push the polygons apart.
    QPointF minTranslation;
};


void projectPolygon(QPointF axis, QPolygonF polygon, qreal &min, qreal &max) ;

qreal intervalDistance(qreal minA, qreal maxA, qreal minB, qreal maxB);
PolygonCollisionResult polygonCollision(QPolygonF polygonA, 
                              QPolygonF polygonB, QPointF velocity);

#endif